package com.tommasoberlose.anotherwidget.network.repository

import com.haroldadmin.cnradapter.NetworkResponseAdapterFactory
import com.tommasoberlose.anotherwidget.global.Preferences
import com.tommasoberlose.anotherwidget.network.api.ApiServices
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory

class HereRepository {

    /* HERE */
    private val apiServiceHere: ApiServices.HereService = getRetrofit().create(ApiServices.HereService::class.java)
    suspend fun getWeather() = apiServiceHere.getWeather(Preferences.weatherProviderApiHere, Preferences.customLocationLat, Preferences.customLocationLon, "observation", true, Preferences.weatherTempUnit != "F")

    companion object {
        private const val BASE_URL_HERE = "https://weather.ls.hereapi.com/weather/1.0/"

        private fun getRetrofit(): Retrofit {
            return Retrofit.Builder()
                .baseUrl(BASE_URL_HERE)
                .addConverterFactory(GsonConverterFactory.create())
                .addCallAdapterFactory(NetworkResponseAdapterFactory())
                .build()
        }
    }

    fun getWeatherIcon(iconName: String, isDaytime: Boolean): String = when(iconName.substringAfter("night_")) {
        "sunny" -> "01"
        "clear" -> "01"
        "mostly_sunny" -> "01"
        "mostly_clear" -> "01"
        "passing_clounds" -> "02"
        "more_sun_than_clouds" -> "02"
        "scattered_clouds" -> "02"
        "partly_cloudy" -> "02"
        "a_mixture_of_sun_and_clouds" -> "03"
        "increasing_cloudiness" -> "03"
        "breaks_of_sun_late" -> "03"
        "afternoon_clouds" -> "03"
        "morning_clouds" -> "03"
        "partly_sunny" -> "03"
        "high_level_clouds" -> "03"
        "decreasing_cloudiness" -> "03"
        "clearing_skies" -> "01"
        "high_clouds" -> "03"
        "rain_early" -> "10"
        "heavy_rain_early" -> "10"
        "strong_thunderstorms" -> "09"
        "severe_thunderstorms" -> "09"
        "thundershowers" -> "11"
        "thunderstorms" -> "11"
        "tstorms_early" -> "11"
        "isolated_tstorms_late" -> "11"
        "scattered_tstorms_late" -> "11"
        "tstorms_late" -> "11"
        "tstorms" -> "11"
        "ice_fog" -> "82"
        "more_clouds_than_sun" -> "03"
        "broken_clouds" -> "03"
        "scattered_showers" -> "10"
        "a_few_showers" -> "10"
        "light_showers" -> "10"
        "passing_showers" -> "10"
        "rain_showers" -> "10"
        "showers" -> "10"
        "widely_scattered_tstorms" -> "11"
        "isolated_tstorms" -> "11"
        "a_few_tstorms" -> "11"
        "scattered_tstorms" -> "11"
        "hazy_sunshine" -> "50"
        "haze" -> "50"
        "smoke" -> "50"
        "low_level_haze" -> "50"
        "early_fog_followed_by_sunny_skies" -> "50"
        "early_fog" -> "82"
        "light_fog" -> "82"
        "fog" -> "82"
        "dense_fog" -> "82"
        //"night_haze"
        //"night_smoke"
        //"night_low_level_haze"
        //"night_widely_scattered_tstorms"
        //"night_isolated_tstorms"
        //"night_a_few_tstorms"
        //"night_scattered_tstorms"
        //"night_tstorms"
        //"night_clear"
        "mostly_cloudy" -> "03"
        "cloudy" -> "04"
        "overcast" -> "04"
        "low_clouds" -> "03"
        "hail" -> "10"
        "sleet" -> "81"
        "light_mixture_of_precip" -> "81"
        "icy_mix" -> "81"
        "mixture_of_precip" -> "81"
        "heavy_mixture_of_precip" -> "81"
        "snow_changing_to_rain" -> "81"
        "snow_changing_to_an_icy_mix" -> "81"
        "an_icy_mix_changing_to_snow" -> "81"
        "an_icy_mix_changing_to_rain" -> "81"
        "rain_changing_to_snow" -> "81"
        "rain_changing_to_an_icy_mix" -> "81"
        "light_icy_mix_early" -> "81"
        "icy_mix_early" -> "81"
        "light_icy_mix_late" -> "81"
        "icy_mix_late" -> "81"
        "snow_rain_mix" -> "81"
        "scattered_flurries" -> "13"
        "snow_flurries" -> "13"
        "light_snow_showers" -> "13"
        "snow_showers" -> "13"
        "light_snow" -> "13"
        "flurries_early" -> "13"
        "snow_showers_early" -> "13"
        "light_snow_early" -> "13"
        "flurries_late" -> "13"
        "snow_showers_late" -> "13"
        "light_snow_late" -> "13"
        //"night_decreasing_cloudiness"
        //"night_clearing_skies"
        //"night_high_level_clouds"
        //"night_high_clouds"
        //"night_scattered_showers"
        //"night_a_few_showers"
        //"night_light_showers"
        //"night_passing_showers"
        //"night_rain_showers"
        //"night_sprinkles"
        //"night_showers"
        //"night_mostly_clear"
        //"night_passing_clouds"
        //"night_scattered_clouds"
        //"night_partly_cloudy"
        //"increasing_cloudiness"
        //"night_afternoon_clouds"
        //"night_morning_clouds"
        //"night_broken_clouds"
        //"night_mostly_cloudy"
        "light_freezing_rain" -> "81"
        "freezing_rain" -> "81"
        "heavy_rain" -> "10"
        "lots_of_rain" -> "10"
        "tons_of_rain" -> "10"
        //"heavy_rain_early" -> "10"
        "heavy_rain_late" -> "10"
        "flash_floods" -> "10"
        "flood" -> "10"
        "drizzle" -> "10"
        "sprinkles" -> "10"
        "light_rain" -> "10"
        "sprinkles_early" -> "10"
        "light_rain_early" -> "10"
        "sprinkles_late" -> "10"
        "light_rain_late" -> "10"
        "rain" -> "10"
        "numerous_showers" -> "10"
        "showery" -> "10"
        "showers_early" -> "10"
        //"rain_early" -> "10"
        "showers_late" -> "10"
        "rain_late" -> "10"
        "snow" -> "13"
        "moderate_snow" -> "13"
        "snow_early" -> "13"
        "snow_late" -> "13"
        "heavy_snow" -> "13"
        "heavy_snow_early" -> "13"
        "heavy_snow_late" -> "13"
        "tornado" -> "80"
        "tropical_storm" -> "09"
        "hurricane" -> "80"
        "sandstorm" -> "50"
        "duststorm" -> "50"
        "snowstorm" -> "13"
        "blizzard" -> "13"
        else -> ""
    } + if (isDaytime) "d" else "n"
}
